#include "../stdafx.h"
#include "DllOutput.h"
#include "../MainHelper.h"
#include "../Resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern Helper m_Helper;

CDllOutput::~CDllOutput()
{
	delete internalOut;
	if (hModOut != NULL)
	{
		FreeLibrary(hModOut);
	}
	hModOut = NULL;
}

CDllOutput::CDllOutput(CString dllPath, HWND hWnd) : OutputInterface(dllPath)
{
	internalOut = new CStandardOutput("", hWnd);
	internalOutMod = internalOut->GetOut_Module();

	if (dllPath.GetLength() != 0)
	{
		// returns a handle to the dll, otherwise NULL
		hModOut = LoadLibrary(dllPath); 
		if (hModOut == NULL)
		{
			AfxMessageBox(m_Helper.MyLoadString(IDS_FAILED_LOAD_OUTPUT) + GetFileName(), MB_OK | MB_ICONEXCLAMATION);
			dllOutMod = internalOutMod;
			SetFilePath("STANDARD");
			hModOut = NULL;
		}
		else
		{
			// returns the address of the dll function, otherwise NULL
			try
			{
				pfun1 = (winampGetOutModule)GetProcAddress(hModOut, "winampGetOutModule");

				// (Dll function address) (function parameters)
				dllOutMod = (pfun1)();
				dllOutMod->hMainWindow = hWnd;

				//Init des fonctions non dclare par celle par dfaut..

				if (dllOutMod->Config == NULL)
					dllOutMod->Config = internalOutMod->Config;
				if (dllOutMod->About == NULL)
					dllOutMod->About = internalOutMod->About;
				if (dllOutMod->Init == NULL)
					dllOutMod->Init = internalOutMod->Init;
				if (dllOutMod->Quit == NULL)
					dllOutMod->Quit = internalOutMod->Quit;
				if (dllOutMod->Open == NULL)
					dllOutMod->Open = internalOutMod->Open;
				if (dllOutMod->Close == NULL)
					dllOutMod->Close = internalOutMod->Close;
				if (dllOutMod->Write == NULL)
					dllOutMod->Write = internalOutMod->Write;
				if (dllOutMod->CanWrite == NULL)
					dllOutMod->CanWrite = internalOutMod->CanWrite;
				if (dllOutMod->IsPlaying == NULL)
					dllOutMod->IsPlaying = internalOutMod->IsPlaying;
				if (dllOutMod->Pause == NULL)
					dllOutMod->Pause = internalOutMod->Pause;
				if (dllOutMod->SetVolume == NULL)
					dllOutMod->SetVolume = internalOutMod->SetVolume;
				if (dllOutMod->SetPan == NULL)
					dllOutMod->SetPan = internalOutMod->SetPan;
				if (dllOutMod->Flush == NULL)
					dllOutMod->Flush = internalOutMod->Flush;
				if (dllOutMod->GetOutputTime == NULL)
					dllOutMod->GetOutputTime = internalOutMod->GetOutputTime;
				if (dllOutMod->GetWrittenTime == NULL)
					dllOutMod->GetWrittenTime = internalOutMod->GetWrittenTime;
			}
			catch (...)
			{
				AfxMessageBox(m_Helper.MyLoadString(IDS_FAILED_LOAD_OUTPUT) + GetFileName(), MB_OK | MB_ICONEXCLAMATION);
				dllOutMod = internalOutMod;
				SetFilePath("STANDARD");
				hModOut = NULL;
			}
		}
	}
	else
	{
		dllOutMod = internalOutMod;
		SetFilePath("STANDARD");
		hModOut = NULL;
	}
	dllOutMod->hDllInstance = hModOut;
}

int CDllOutput::GetVersion()	   // module version (OUT_VER)
{
	return dllOutMod->version;
}

TCHAR* CDllOutput::GetDescription() 	// description of module, with version string
{
	return dllOutMod->description;
}
int CDllOutput::GetId() 		   // module id. each input module gets its own. non-nullsoft modules should
{
	return dllOutMod->id;
}


void CDllOutput::SethMainWindow(HWND hWnd)  	// winamp's main window (filled in by winamp)
{
	dllOutMod->hMainWindow = hWnd;
	internalOut->SethMainWindow(hWnd);
}
void CDllOutput::SethDllInstance(HINSTANCE hDLL)	// DLL instance handle (filled in by winamp)
{
	dllOutMod->hDllInstance = hDLL;
}

void CDllOutput::Config(HWND hwndParent) // configuration dialog
{
	dllOutMod->Config(hwndParent);
}

void CDllOutput::About(HWND hwndParent)  // about dialog
{
	dllOutMod->About(hwndParent);
}

void CDllOutput::Init() 	  // called when loaded
{
	dllOutMod->Init();
}

void CDllOutput::Quit() 	  // called when unloaded
{
	dllOutMod->Quit();
}

int CDllOutput::Open(int samplerate, int numchannels, int bitspersamp, int bufferlenms, int prebufferms)
	// returns >=0 on success, <0 on failure
	// NOTENOTENOTE: bufferlenms and prebufferms are ignored in most if not all output plug-ins. 
	//    ... so don't expect the max latency returned to be what you asked for.
	// returns max latency in ms (0 for diskwriters, etc)
	// bufferlenms and prebufferms must be in ms. 0 to use defaults. 
	// prebufferms must be <= bufferlenms
{
	return dllOutMod->Open(samplerate, numchannels, bitspersamp, bufferlenms, prebufferms);
}

void CDllOutput::Close()  // close the ol' output device.
{
	dllOutMod->Close();
}

int CDllOutput::Write(char* buf, int len)
	// 0 on success. Len == bytes to write (<= 8192 always). buf is straight audio data. 
	// 1 returns not able to write (yet). Non-blocking, always.
{
	return dllOutMod->Write(buf, len);
}

int CDllOutput::CanWrite() // returns number of bytes possible to write at a given time. 
	// Never will decrease unless you call Write (or Close, heh)
{
	return dllOutMod->CanWrite();
}

int CDllOutput::IsPlaying() // non0 if output is still going or if data in buffers waiting to be
	// written (i.e. closing while IsPlaying() returns 1 would truncate the song
{
	return dllOutMod->IsPlaying();
}

int CDllOutput::Pause(int pause) // returns previous pause state
{
	return dllOutMod->Pause(pause);
}


void CDllOutput::SetVolume(int volume) // volume is 0-255
{
	dllOutMod->SetVolume(volume);
}
void CDllOutput::SetPan(int pan) // pan is -128 to 128
{
	dllOutMod->SetPan(pan);
}

void CDllOutput::Flush(int t) // flushes buffers and restarts output at time t (in ms) 
	// (used for seeking)
{
	dllOutMod->Flush(t);
}

int CDllOutput::GetOutputTime() // returns played time in MS
{
	return dllOutMod->GetOutputTime();
}
int CDllOutput::GetWrittenTime() // returns time written in MS (used for synching up vis stuff)
{
	return dllOutMod->GetWrittenTime();
}

Out_Module* CDllOutput::GetOut_Module()
{
	return dllOutMod;
}