// StandardOutputConfig.cpp : fichier d'implmentation
//

#include "../stdafx.h"
#include "resource.h"   	// main symbols
#include "../MainHelper.h"
#include "audio.h"
#include <assert.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define DEFAULT_BLOCK_SIZE_INDEX 7
#define DEFAULT_BLOCK_COUNT_INDEX 6
#define WM_UPDATE_DISPLAY (WM_USER + 3)

/*
 * dialog temporary config
 */
static config_t dialog_config;

/*
 * conversion tables for sliders/values
 */ 
static int size_table[] =
{
	65536, 49170, 32768, 24576, 16384, 8192, 6144, 4096, 2048, 1024
};
static int count_table[] =
{
	50, 40, 35, 30, 25, 20, 15, 10, 5, 2
};

/*
 * convert slider value to a block size 
 */
static int index_to_block_size(int value)
{
	if (value < 0)
		return size_table[0];
	if (value > 9)
		return size_table[9];

	return size_table[value];
}

/*
 * convert slider value to block count
 */
static int index_to_block_count(int value)
{
	if (value < 0)
		return count_table[0];
	if (value > 9)
		return count_table[9];

	return count_table[value];
}

/* 
 * convert block count to slider value
 */
static int block_count_to_index(int value)
{
	int i;

	for (i = 0; i < sizeof(count_table) / sizeof(count_table[0]); i++)
	{
		if (value == count_table[i])
			return i;
	}

	return DEFAULT_BLOCK_COUNT_INDEX;
}

/*
 * convert block size to slider value
 */
static int block_size_to_index(int value)
{
	int i;

	for (i = 0; i < sizeof(size_table) / sizeof(size_table[0]); i++)
	{
		if (value == size_table[i])
			return i;
	}

	return DEFAULT_BLOCK_SIZE_INDEX;
} 

/*
 * dialog procedure for the configuration
 */ 
BOOL CALLBACK config_dialog_proc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	stats_t stats;
	static int timer_slow = 0;
	int i;
	int j;
	LPPSHNOTIFY lppsn;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		{
			/*
																									 * initialise controls and set their respective ranges etc.
																									 */
			get_config(&dialog_config);

			/*
																									 * fill the combo with the wave devices
																									 */
			i = (int)waveOutGetNumDevs();
			SendDlgItemMessage(hDlg, IDC_COMBO_DEVICE, CB_ADDSTRING, 0, (LONG)_T("Wave Mapper"));
			for (j = 0; j < i; j++)
			{
				WAVEOUTCAPS woc;
				waveOutGetDevCaps(j, &woc, sizeof(WAVEOUTCAPS));
				SendDlgItemMessage(hDlg, IDC_COMBO_DEVICE, CB_ADDSTRING, 0, (LONG)woc.szPname);
			}

			/*
																									 * update the controls
																									 */
			SendMessage(hDlg, WM_UPDATE_DISPLAY, 0, 0);

			/*
																									 * begin stats update timer
																									 */
			timer_slow = 0;
			SetTimer(hDlg, 101, 10, NULL);
			break;
		}
	case WM_VSCROLL:
		if (LOWORD(wParam) >= TB_THUMBTRACK && LOWORD(wParam) <= TB_ENDTRACK)
		{
			TCHAR buf[64];

			dialog_config.block_size = index_to_block_size(SendDlgItemMessage(hDlg, IDC_SLIDER_SIZE, TBM_GETPOS, 0, 0));
			dialog_config.block_count = index_to_block_count(SendDlgItemMessage(hDlg, IDC_SLIDER_COUNT, TBM_GETPOS, 0, 0));

			if ((HWND)lParam == GetDlgItem(hDlg, IDC_SLIDER_SIZE))
			{
				wsprintf(buf, _T("%d kB"), dialog_config.block_size >> 10);
				SendDlgItemMessage(hDlg, IDC_LABEL_SIZE, WM_SETTEXT, 0, (LPARAM)buf);
			}
			else
			{
				wsprintf(buf, _T("%d"), dialog_config.block_count);
				SendDlgItemMessage(hDlg, IDC_LABEL_COUNT, WM_SETTEXT, 0, (LPARAM)buf);
			}
		}
		break;
	case WM_TIMER:
		{
			TCHAR buffer[64];

			get_statistics(&stats);

			wsprintf(buffer, _T("%d kB (%d x %d kB)"), stats.total_buffer_size / 1024, stats.block_count, stats.block_size / 1024);
			SendDlgItemMessage(hDlg, IDC_LABEL_TOTAL, WM_SETTEXT, 0, (LPARAM)buffer);
			wsprintf(buffer, _T("%d/%d"), stats.block_usage, stats.block_count);
			SendDlgItemMessage(hDlg, IDC_LABEL_BLOCKS, WM_SETTEXT, 0, (LPARAM)buffer);

			if (stats.active)
			{
				wsprintf(buffer, _T("%d"), stats.bytes_written);
				SendDlgItemMessage(hDlg, IDC_LABEL_WRITTEN, WM_SETTEXT, 0, (LPARAM)buffer);
				wsprintf(buffer, _T("%d"), stats.bytes_played);
				SendDlgItemMessage(hDlg, IDC_LABEL_PLAYED, WM_SETTEXT, 0, (LPARAM)buffer);
				wsprintf(buffer, _T("%d ms"), stats.start_position);
				SendDlgItemMessage(hDlg, IDC_LABEL_START, WM_SETTEXT, 0, (LPARAM)buffer);
				wsprintf(buffer, _T("%d ms"), stats.latency);
				SendDlgItemMessage(hDlg, IDC_LABEL_LATENCY, WM_SETTEXT, 0, (LPARAM)buffer);
				wsprintf(buffer, _T("%d Hz, %dbit, %dch"), stats.format.samplerate, stats.format.bitspersamp, stats.format.numchannels);
				SendDlgItemMessage(hDlg, IDC_LABEL_FORMAT, WM_SETTEXT, 0, (LPARAM)buffer);
				SendDlgItemMessage(hDlg, IDC_LABEL_CONV, WM_SETTEXT, 0, (LPARAM)(stats.format.conv ? _T("Active") : _T("Inactive")));
				SendDlgItemMessage(hDlg, IDC_PROGRESS_USAGE, PBM_SETRANGE32, 0, stats.total_buffer_size);
				SendDlgItemMessage(hDlg, IDC_PROGRESS_USAGE, PBM_SETPOS, stats.total_buffer_usage, 0);

				if (timer_slow)
				{
					KillTimer(hDlg, 101);
					SetTimer(hDlg, 101, 10, NULL);
					timer_slow = 0;
				}
			}
			else
			{
				SendDlgItemMessage(hDlg, IDC_LABEL_WRITTEN, WM_SETTEXT, 0, (LPARAM)_T("0"));
				SendDlgItemMessage(hDlg, IDC_LABEL_PLAYED, WM_SETTEXT, 0, (LPARAM)_T("0"));
				SendDlgItemMessage(hDlg, IDC_LABEL_START, WM_SETTEXT, 0, (LPARAM)_T("0 ms"));
				SendDlgItemMessage(hDlg, IDC_LABEL_LATENCY, WM_SETTEXT, 0, (LPARAM)_T("(Inactive)"));
				SendDlgItemMessage(hDlg, IDC_LABEL_FORMAT, WM_SETTEXT, 0, (LPARAM)_T("(Inactive)"));
				SendDlgItemMessage(hDlg, IDC_LABEL_CONV, WM_SETTEXT, 0, (LPARAM)_T("(Inactive)"));
				SendDlgItemMessage(hDlg, IDC_PROGRESS_USAGE, PBM_SETPOS, 0, 0);

				if (!timer_slow)
				{
					KillTimer(hDlg, 101);
					SetTimer(hDlg, 101, 10, NULL);
					timer_slow = 1;
				}
			}
			break;
		}
		/*
													 * update display
													 */
	case WM_UPDATE_DISPLAY:
		SendDlgItemMessage(hDlg, IDC_SLIDER_SIZE, TBM_SETRANGE, TRUE, MAKELONG(0, 9));
		SendDlgItemMessage(hDlg, IDC_SLIDER_COUNT, TBM_SETRANGE, TRUE, MAKELONG(0, 9));
		/*
													 * set up controls based on config
													 */
		SendDlgItemMessage(hDlg, IDC_COMBO_DEVICE, CB_SETCURSEL, dialog_config.device + 1, 0);
		SendDlgItemMessage(hDlg, IDC_SLIDER_SIZE, TBM_SETPOS, TRUE, block_size_to_index(dialog_config.block_size));
		SendDlgItemMessage(hDlg, IDC_SLIDER_COUNT, TBM_SETPOS, TRUE, block_count_to_index(dialog_config.block_count));
		SendDlgItemMessage(hDlg, IDC_CHECK_VOLUME, BM_SETCHECK, dialog_config.volume_enabled ? BST_CHECKED : BST_UNCHECKED, 0);
		SendDlgItemMessage(hDlg, IDC_CHECK_BALANCE, BM_SETCHECK, dialog_config.reverse_balance ? BST_CHECKED : BST_UNCHECKED, 0);
		SendDlgItemMessage(hDlg, IDC_CHECK_DOWNMIX, BM_SETCHECK, dialog_config.pcmconv_enabled ? BST_CHECKED : BST_UNCHECKED, 0);

		/*
													 * update slider labels. yes this is a hack.
													 */
		SendMessage(hDlg, WM_VSCROLL, TB_ENDTRACK, (LPARAM)GetDlgItem(hDlg, IDC_SLIDER_SIZE));
		SendMessage(hDlg, WM_VSCROLL, TB_ENDTRACK, (LPARAM)GetDlgItem(hDlg, IDC_SLIDER_COUNT));
		break;
	case WM_NOTIFY:
		lppsn = (LPPSHNOTIFY)lParam;
		switch (lppsn->hdr.code)
		{
		case PSN_APPLY :
			/*
																									   * grab some various control states and save them back to our config struct
																									   */
			dialog_config.volume_enabled = SendDlgItemMessage(hDlg, IDC_CHECK_VOLUME, BM_GETCHECK, 0, 0) == BST_CHECKED ? 1 : 0;
			dialog_config.reverse_balance = SendDlgItemMessage(hDlg, IDC_CHECK_BALANCE, BM_GETCHECK, 0, 0) == BST_CHECKED ? 1 : 0;
			dialog_config.device = SendDlgItemMessage(hDlg, IDC_COMBO_DEVICE, CB_GETCURSEL, 0, 0) - 1;
			dialog_config.pcmconv_enabled = SendDlgItemMessage(hDlg, IDC_CHECK_DOWNMIX, BM_GETCHECK, 0, 0) == BST_CHECKED ? 1 : 0;

			/*
																													* push the config struct back at the plugin. changes are updated when playback is next
																													* started/restarted.
																													*/
			set_config(&dialog_config);
			config_save(&dialog_config);
			break;
		case PSN_QUERYCANCEL:
			KillTimer(hDlg, 101);
			EndDialog(hDlg, LOWORD(wParam));
			break;
		}
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
		case IDCANCEL:
		case IDC_DEFAULTS:
			if (MessageBox(hDlg, _T("Are you sure you want to reset to the default settings?"), _T("waveOut Configuration"), MB_YESNO | MB_ICONQUESTION) == IDYES)
			{
				config_set_defaults(&dialog_config);
				SendMessage(hDlg, WM_USER + 3, 0, 0);
			}
			break;
		default:
			return FALSE;
		}
	default:
		return FALSE;
	}

	return TRUE;
}

